package gs.test.v0.api

import cats.Show

/** Opaque type that represents the number of iterations a test should run. This
  * value must be at least `1` (the default). To ignore a test, use the
  * [[Test.Marker.Ignored]] marker.
  */
opaque type TestIterations = Int

object TestIterations:

  def One: TestIterations = 1

  /** Validate and instantiate a new [[TestIterations]] instance.
    *
    * @param candidate
    *   The candidate value. Must be 1 or greater.
    * @return
    *   The new [[TestIterations]], or an error if an invalid input is given.
    */
  def apply(candidate: Int): TestIterations =
    if candidate < 1 then
      throw new IllegalArgumentException(
        s"Tests must iterate at least once. Received candidate '$candidate'."
      )
    else candidate

  given CanEqual[TestIterations, TestIterations] = CanEqual.derived

  given Show[TestIterations] = iters => iters.toString()

  extension (iters: TestIterations) def toInt(): Int = iters

end TestIterations
