package gs.test.v0.runtime.engine

import cats.effect.Async
import cats.effect.Ref
import cats.syntax.all.*
import java.util.concurrent.TimeUnit
import scala.concurrent.duration.FiniteDuration

final class EngineStats[F[_]: Async](
  val overallDuration: Ref[F, FiniteDuration],
  val countSeen: Ref[F, Long],
  val countSucceeded: Ref[F, Long],
  val countFailed: Ref[F, Long],
  val countIgnored: Ref[F, Long]
)

object EngineStats:

  def initialize[F[_]: Async]: F[EngineStats[F]] =
    for
      duration  <- Ref.of(FiniteDuration(0L, TimeUnit.NANOSECONDS))
      seen      <- Ref.of(0L)
      succeeded <- Ref.of(0L)
      failed    <- Ref.of(0L)
      ignored   <- Ref.of(0L)
    yield new EngineStats[F](
      overallDuration = duration,
      countSeen = seen,
      countSucceeded = succeeded,
      countFailed = failed,
      countIgnored = ignored
    )

end EngineStats
